import java.awt.*;
import java.awt.event.*;
import javax.naming.NameAlreadyBoundException;
import javax.swing.*;

/*
 * Authentification module, with a graphical component
 */
public class Authentification extends JPanel implements ActionListener, KeyListener {
	private static final long serialVersionUID = 2759476966928186665L;

	// Graphical components
	private JTextField clientName;
	private JTextField serverName;
	private JTextField serverPort;
	private JTextField clientPort;
	private JButton connectButton = new JButton( "I CAN HAZ CONNEKSHIUN" );
	private JLabel errorlabel = new JLabel();
	
	// Logical interactions
	private ServerInteraction server;
	private Screen screen;
	private Game game;
		
	/*
	 * Creates an authentification form and module  
	 */
	public Authentification(Screen _screen, ServerInteraction _serverInteraction, Game _game) {
		// Logical components
		screen = _screen;
		server = _serverInteraction;
		game = _game;
		
		// Fields initialisation
		clientName = new JTextField(Client.name, 15);
		serverName = new JTextField(server.host, 15);
		serverPort = new JTextField(Integer.toString(server.serverPort), 5);
		clientPort = new JTextField(Integer.toString(Client.port), 5);
		
		Font police = new Font("Tahoma", Font.ITALIC, 12 );
		errorlabel.setFont(police);
		errorlabel.setForeground(new Color(133,3,47));
		
		// Adding subcomponents
		this.add(label("!!!!! CONNEKSHIUN !!!!!", 20 ));
		this.add(label("Ur naymz :", 12 ));	
		this.add (clientName);
		this.add(label("Ur serverz wif portz :", 12 ));
		this.add (serverName);
		this.add (serverPort);
		this.add(label("U can haz portz :", 12 ));
		this.add (clientPort);
		this.add (connectButton);
		this.add (errorlabel);
		this.setBackground(new Color(173,143,197));
		this.setBorder(BorderFactory.createRaisedBevelBorder());

		// Adding listeners
		connectButton.addActionListener(this);
		clientName.addKeyListener(this);
		serverName.addKeyListener(this);
		serverPort.addKeyListener(this);
	}

	/*
	 * Provides a text label with the given content
	 */
	private JLabel label(String content, int size) {
		JLabel label = new JLabel();
		label.setText(content);
		Font police = new Font("Tahoma", Font.BOLD, size );
		label.setFont(police);
		label.setForeground(new Color(73,43,97));
		label.setAlignmentY(Component.LEFT_ALIGNMENT);
		return label;
	}

	/*
	 * Attempts to connect to the server with the informations given by the form
	 */
	private void tryConnect() {
		server.host = serverName.getText();
		server.serverPort = Integer.parseInt(serverPort.getText());
		Client.port = Integer.parseInt(clientPort.getText());
		Client.name = clientName.getText(); 
		try {
			server.initialize();
			game.listenLarboard();
	    	errorlabel.setText("");	
			screen.displayChat();
		}
		catch (NameAlreadyBoundException e) {
    		errorlabel.setText("ALL UR NAMZ R BELONG TO ANOTHR CASTL !");
    		setServerEditable(false); 
    		// The socket is initialized, we must not create another one.
		}
		catch (Exception e) {
    		errorlabel.setText("O NOES ! I DOESNT CONNEKT RITE TO SERVR =(");	
    		setServerEditable(true);
		}		
	}

	/*
	 * Sets whether the server and serverPort fields are editable
	 */
	public void setServerEditable(boolean b) {
		serverName.setEditable(b);
		serverPort.setEditable(b);
	}
	
	/*
	 * Event handler for the validation of the form with the button
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == connectButton) {
			tryConnect();
	    }
	} 
	
	/*
	 * Event handler for the validation of the form with the key
	 */
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        
        if (key == KeyEvent.VK_ENTER) {
        	tryConnect();
        }
    }
    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
}
