import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/*
 * Chat module with a graphical component
 */
public class Chat extends JPanel implements ActionListener, KeyListener {
	private static final long serialVersionUID = 2659476966928186665L;
	
	// Graphical components
	private JTextField line = new JTextField("", 50);
	private JButton sendButton = new JButton( "SAYZ !" );
	private static JTextPane chatPanel = new JTextPane();
	static JScrollPane scrollPane;
	
	// Logical interactions
	private ServerInteraction server;
	
	Chat(ServerInteraction _serverInteraction) {
		server = _serverInteraction;

		chatPanel.setPreferredSize(new Dimension(700,200));
		scrollPane = new JScrollPane (chatPanel);
    	
    	this.emptyChat();
		this.add(scrollPane);
		this.add(line);
		this.add(sendButton);
		this.setOpaque(false);
		
		sendButton.addActionListener(this);
		line.addKeyListener(this);
		

	}
	
	/*
	 * Add a regular line to the chat
	 */
	public void addLine(String from, String msg) {
    	SimpleAttributeSet s_name = new SimpleAttributeSet();
    	SimpleAttributeSet s_msg = new SimpleAttributeSet();
    	Document doc = chatPanel.getDocument();
    	
    	// Determines the display style
	    StyleConstants.setBold(s_name, true);
	    // Myself is blue
	    if (from.equals(Client.name)) { 
	    	StyleConstants.setForeground(s_name, Color.BLUE);
	    }
	    // Leader is pink with a kirby
	    if (from.equals(server.players.leader().name)){
        	from = "(>*-*)> " + from;
            StyleConstants.setForeground(s_name, Color.MAGENTA);
	    }
	    
	    // /me gives an italic text
	    if (msg.length() > 4 && msg.substring(0,4).equals("/me ")) {
	    	msg = msg.substring(3);
	    	StyleConstants.setItalic(s_name, true);
	    	StyleConstants.setItalic(s_msg, true);
	    }
	    else {
	    	msg = " : " + msg;
	    }

	    // Actual display of the text
    	try {
			doc.insertString(doc.getLength(), from, s_name);
	    	doc.insertString(doc.getLength(), msg + "\n", s_msg);	   
		} catch (BadLocationException e) {
			System.err.println("[NOES CAN PRINT IN TEXTBLOK =(]\n" + e);
		}  
		
		// Scrolling the scrollbar
        scrollPane.getVerticalScrollBar().setValue(scrollPane.getVerticalScrollBar().getMaximum());
	}

	/*
	 * Clear the chat and displays a little welcome message
	 */
	public void emptyChat() {
    	SimpleAttributeSet s_init = new SimpleAttributeSet();
    	StyleConstants.setItalic(s_init, true);
    	
		try {
    		chatPanel.setText("");
    		chatPanel.getDocument().insertString(0, "Welkum in teh server " + server.host + ":" + server.serverPort + " ! U can haz a lolchat w8tin for teh tetrish to start !!!\n", s_init);
		} catch (BadLocationException e) {
			System.err.println("[NOES CAN PRINT IN TEXTBLOK =(]\n" + e);
		}  
	}
	
	/*
	 * Sends the current line to the server and resets it
	 */
	private void sendLine(){
		if (line.getText().length() == 0){
			line.setText(" ");
		}
		server.chat(line.getText());
		line.setText("");
	}
	
	/*
	 * Event handler for the validation of a message with the button
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == sendButton) {
			sendLine();
		}
	} 

	/*
	 * Event handler for the validation a message with the key
	 */
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        
        if (key == KeyEvent.VK_ENTER) {
			sendLine();
        }
    }
    public void keyTyped(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
}
