import java.awt.Color;

import javax.swing.JPanel;

/*
 * Logic and graphic management for the game
 */
public class Game extends JPanel {
	private static final long serialVersionUID = 4799416209402503264L;

	// Game seed sent by the server
	public int seed;
	private int respawn = 3000;

	// Neighbours
	public PlayerInteraction p_larboard;
	public PlayerInteraction p_starboard;
	
	// Logical elements of the game
	public Board board;
	private NeighbourBoard starboard, larboard;
	private Boolean isStarted = false;
	
	//  Color global settings
	public static Color BackgroundColor = new Color(185,125,185);
	
	/*
	 *  Handles the creation of the graphical game pannel
	 */
	public Game() {
		this.setOpaque(false);
		this.setLayout(null);
	}

	/*
	 * Gets ready to listen for a connexion from starboard
	 */
	public void listenLarboard() {
		p_larboard = new PlayerInteraction(new Player("","",0), PlayerInteraction.Role.LARBOARD, this);
	}
	
	/*
	 * Closes the listening on starboard
	 */
	public void closesLarboard() {
		p_larboard.close();
		p_larboard = null;
	}
	
	/*
	 *  Handles the beginning of the game when the players are ready, must start only once
	 */
	public void tryStart() {
		if (p_larboard.isReady && p_starboard.isReady && !isStarted){
			this.setVisible(true);
			isStarted = true;
		
			PiecesManager piecesManager = new PiecesManager(this, seed);

	        Thread piecesThread = new Thread(piecesManager);
	        piecesThread.setName("MemeTris - Pieces manager");
	        piecesThread.setPriority(Thread.MIN_PRIORITY);
	        piecesThread.start();

	        ScoreBoard scoreBoard = new ScoreBoard();
			this.add(scoreBoard);

			board = new Board(piecesManager, this, scoreBoard);
			this.add(board);
			
			starboard = new NeighbourBoard(600,0, p_starboard.player.name);
			this.add(starboard);
			
			larboard = new NeighbourBoard(600,290, p_larboard.player.name);
			this.add(larboard);

	        board.setFocusable(true);
			board.requestFocusInWindow();
		}
	}
	

	/* 
	 * Handles a tic of the clock
	 */
	public void tic(){
		board.tic();
	}
	
	/*
	 * Broadcast own board state to neighbours
	 */
	public void broadcastBoard(Piece.Type[][] board){
		p_larboard.sendBoard(board);
		p_starboard.sendBoard(board);
	}
	
	/*
	 * Updates the display of a received neighbour board
	 */
	public void updateNeighbourBoard(PlayerInteraction.Role role, Piece.Type[][] _board){
		if (larboard != null && role == PlayerInteraction.Role.LARBOARD) {
			larboard.updateContent(_board);
		}
		else if (starboard != null) {
			starboard.updateContent(_board);			
		}
		 refreshNeighboursdName();
	}

	/*
	 * Handles the transmission of penalties
	 */
	public void sendPenalties(int n){
		if (n>0 && ! p_starboard.player.name.equals(Client.name) ) {
			p_starboard.sendPenalties(n);
		}
	}

	/*
	 * Handles the taking of penalties
	 */
	public void takePenalties(int n){
		board.takePenalties(n);
	}
	
	/*
	 * Handles the loss
	 */
	public void loose(){
		if (isStarted)
		{
			p_starboard.loose(p_larboard.player);
			p_larboard.loose(p_starboard.player);
			
			// Resets the game
		     long t0, t1;
		     t0 =  System.currentTimeMillis();
		     do{
		    	 t1 = System.currentTimeMillis();
		    }
		    while (t1 - t0 < respawn);
	
		    System.out.println("ALL SYSTMZ SHUTDOWN, PREPARE TO HAZ ANOTHER TETRISH");
			p_starboard.close();
		    p_larboard.close();
		    this.setVisible(false);
		    Client.restart();		
		}
	}
	

	/*
	 * Reprints the name of the neighbours
	 */
	public void refreshNeighboursdName(){
		if (starboard != null && larboard != null) {
			larboard.updateName(p_larboard.player.name);
			starboard.updateName(p_starboard.player.name);		
		}
	}
}
