import java.awt.*;

/*
 * Graphical module for grids
 */
public class Grid {
	// Fixed parameters
	private int squareSize = 26;
	private int padding = 10;
	private int extraPadding = 0;
	private Color color = Game.BackgroundColor;
	
	// Grid settings
	private int width;
	private int height;
	private Piece.Type[][] content;	

	/*
	 *  Creates an empty grid
	 */
	public Grid(int w, int h) {
		width = w;
		height = h;
		content = new Piece.Type[height][width];

		for(int x=0;x<width;x++){
			for(int y=0;y<height;y++){
		        content[y][x] = Piece.Type.empty;
			}		
		}
	}
	
	/*
	 *  Creates an empty grid with custom parameters
	 */
	public Grid(int w, int h, int size, int _padding, int _extraPadding, Color _color) {
		squareSize = size;
		color = _color;
		padding = _padding;
		extraPadding = _extraPadding;
		width = w;
		height = h;
		content = new Piece.Type[height][width];

		for(int x=0;x<width;x++){
			for(int y=0;y<height;y++){
		        content[y][x] = Piece.Type.empty;
			}		
		}
	}

	/*
	 * Gets a new content for the grid
	 */
	public void setContent(Piece.Type[][] _content) {
		content = _content;
	}	

	/*
	 * Print the grid on a graphic layout
	 */
    public void print(Graphics g){
        Graphics2D g2d = (Graphics2D)g;    
        g.setColor(color);   
        g.fillRect(0, 0, squareSize * width+2*padding, squareSize * height+2*padding+ extraPadding);
        

		for(int x=0;x<width;x++){
			for(int y=0;y<height;y++){
	            GradientPaint gp;
	            
	            // Picks the color of the current piece
	            switch (content[y][x]) {
		            case I :  gp = Board.ColorI;  break;
		            case J :  gp = Board.ColorJ;  break;
		            case L :  gp = Board.ColorL;  break;
		            case O :  gp = Board.ColorO;  break;
		            case S :  gp = Board.ColorS;  break;
		            case T :  gp = Board.ColorT;  break;
		            case Z :  gp = Board.ColorZ;  break;
		            case P :  gp = Board.ColorP;  break;
		            default: gp = Board.ColorEmpty; break;
	            }
	             
	            g2d.setPaint(gp);
	            
	            // Draws the rectangle a little smaller so as to see the grid
	            g2d.fillRect(squareSize*x+padding+1, squareSize*y+padding+1, squareSize-1, squareSize-1);
			}		
		}
    } 
}
