import java.awt.event.*;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

/*
 * Logic for pieces generation
 */
public class PiecesManager implements Runnable, ActionListener  {
	private int DownTimeOut = 1000;
	private Timer DownTimer;
	Game game;

	// Pseudo-random generator initialized at the seed
	private long random;

	/* 
	 * Initializes the pseudorandom generator with the seed
	 */
	public PiecesManager(Game _game, int seed) {
		game = _game;
		random = seed;
		
		DownTimer = new Timer(DownTimeOut,this);
		DownTimer.start();
	}

    /* 
     * Decreases the timeout when leveling up
     */
    public void levelup(){
		DownTimer.setDelay(DownTimeOut);
    	DownTimeOut *= 0.9;	
    }
    
    /* 
     * Next step of pseudorandom generation
     */
    private long nextInt(){
    	return  ((15731 * random) % 32003);	
    }

	/*
	 * Transforms a given int to a piece type
	 */
	public Piece.Type intToType(long i) {
    	Piece.Type type = Piece.Type.empty;
    	
        switch ((int)(i % 7)) {
            case 0 :  type = Piece.Type.I;  break;
            case 1 :  type = Piece.Type.J;  break;
            case 2 :  type = Piece.Type.L;  break;
            case 3 :  type = Piece.Type.O;  break;
            case 4 :  type = Piece.Type.S;  break;
            case 5 :  type = Piece.Type.T;  break;
            case 6 :  type = Piece.Type.Z;  break;
        }
        
        return type;
	}
	
    /*
     * Updates the speudorandom generator for a round
     */
    public void newType(){
        random =  nextInt();
    }

    /*
     * Returns the current piece type
     */
    public Piece.Type current(){
		return intToType(random);
    }
    
    /*
     * Returns the next piece type to come
     */
    public Piece.Type nextType(){
		return intToType(nextInt());
    }
    
    
    
    
    
    /*
     * Handles any action of the thread
     */
	public void run() {
	}
	
	/*
	 * Stops the timer
	 */
	public void stop() {
		DownTimer.stop();
	}	

	/*
	 * Action listener dealing with the timer
	 */
	public void actionPerformed(ActionEvent e) {
		// Acts upon the board
		if (e.getSource() == DownTimer) {	
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { game.tic(); }
			};
			SwingUtilities.invokeLater(doWorkRunnable);
	    }
	 }
}
