import java.io.IOException;
import java.net.ServerSocket;

/*
 * Handles the peer to peer interactions between players
 */
public class PlayerInteraction {
	public static enum Role {
	    LARBOARD, STARBOARD 
	}
	
	public Player player;
	public Game game;
	public Boolean isReady = false;
	public Boolean active = true;
	
	private Role role;
	private Connection connection;
	private PlayerMonitor monitor;
	public ServerSocket serverSocket;
	
	/*
	 * Creates the thread that will establish the interaction with the given player according to their respective position.
	 */
	PlayerInteraction(Player _player, Role _role, Game _game){
		player = _player;
		role = _role;
		game = _game;
		
		monitor = new PlayerMonitor(this, role);
	    Thread monitorThread = new Thread(monitor);
	    monitorThread.setName("MemeTris - Player peer to peer monitor on " + player.name + "[" + player.ip + "]");
	    monitorThread.setPriority(Thread.MIN_PRIORITY);
	    monitorThread.start();

		System.out.println("! MONITORING INISHIALIZED AT " + role + " (WIF " + player.name + " ON IP " + player.ip + ")");
	}

	/* 
	 * Handles the closing of the connection
	 */
	public void close() {
		active = false;
		
		if (role == PlayerInteraction.Role.STARBOARD) { 
			connection.close();
		}
		else {
			try {
				serverSocket.close();
				System.out.println("LARBOAR SERVER CLOSED SUCCESSFULLY");
			} catch (IOException e) {
			}
		}
	}

	/* 
	 * Used by the monitoring thread to establish connection 
	 * necessary because we simultaneously launch all connections and all receptions
	 */
	public void setConnection(Connection _connection) {
		connection = _connection;	
	}
	
	/*
	 * Introduces the client to their new neighbour
	 */
	synchronized public void introduction() {
		connection.send("HOHAI!!! I IZ " + Client.name);
	}
	
	/*
	 * Answers the introduction
	 */
	synchronized public void answerIntroduction() {
		connection.send("GRT! ME WUZ WAITIN 4 U TO PLAY WIF!! MAH NAYMZ " + Client.name);	
	}

	/*
	 * Transmits the current board state to the player
	 */
	synchronized public void sendBoard(Piece.Type[][] board){
		connection.send("MAH BORD LOOKZ LIEK DIS");
		
		int height = board.length;
		int width = board[0].length;

		for(int y=0;y<height;y++){
    		String line = "";
        	for(int x=0;x<width;x++){

	            switch (board[y][x]) {
		            case I :  line += "i";  break;
		            case J :  line += "j";  break;
		            case L :  line += "l";  break;
		            case O :  line += "o";  break;
		            case S :  line += "s";  break;
		            case T :  line += "t";  break;
		            case Z :  line += "z";  break;
		            case P :  line += "p";  break;
		            default: line += "."; break;
	            }
	             
			}	
			connection.send(line);
		}
		connection.send("");
	}

	/*
	 * Sends n penalties to the player
	 */
	synchronized public void sendPenalties(int n){
		connection.send("YO DAWG, I HEARD U LIEK PENALTIEZ! HERE IZ " + n + " ONES 4 U!1!");
	}

	/*
	 * Transmits the role of the player
	 */
	public Role getRole(){
		return role;
	}
	
	/*
	 * Gets an incoming message or a null message if the buffer is empty
	 */
 	public String getIn() {
		return connection.getIn();
	}

 	/*
 	 * Transmits a kthxbye message to the other player
 	 */
 	synchronized public void kthxbye(){
 		connection.send("KTHXBYE");
 	}
 	
	/*
	 * Handles the communication after the loss
	 */
 	synchronized public void loose(Player replace){
		String antirole = "LARBOARD";
		if (role == Role.LARBOARD) {
			antirole = "STARBOARD";
		}
		connection.send("I LOST TEH GAME, ALL YUR " + antirole + " ARE BELONG TO " + replace.ip + ":" + replace.port + " NAO");
	}
}
