import java.io.IOException;
import java.net.*;
import javax.swing.SwingUtilities;


/*
 * Thread handling reception of peer to peer messages
 * Also handles the initialization of peer to peer connections
 */
public class PlayerMonitor implements Runnable {
	private PlayerInteraction peer;
	private PlayerInteraction.Role role;


	/* 
	 * Fills the monitor with the required parameters
	 */
	PlayerMonitor (PlayerInteraction interaction, PlayerInteraction.Role _role) {
		peer = interaction;
		role = _role;

	}

	/*
	 * Initializes the connection to the neighbour 
	 * Required because all the connections and receptions will be simultaneous when the tetris start
	 */
	public void initializesConnexion() throws Exception {
		// Connection to larboard : we initialize connection
		if (role == PlayerInteraction.Role.STARBOARD) { 
			peer.setConnection(new Connection(peer.player.ip, peer.player.port));
		}
		
		// Connection from starboard : we receive connection
		else { 
			try {
				System.out.println("! STARBOARD REPORTING SIR !!! I IZ NAO LISSENIN ON PORT " + Client.port + " ! EVERYTHIN IN ORDERZ !!1!");
				
				peer.serverSocket = new ServerSocket(Client.port);
				peer.serverSocket.setSoTimeout(Connection.getTimeout()); // uses the same timeout than socket connections
				
				boolean connectionEstablished = false;
			    while (! connectionEstablished) {
			    	if (! peer.active){
						throw new Exception("Connekshiun shut down");
					}
			    	
					try {
			    	Socket incomingConnexion = peer.serverSocket.accept();
			    	InetAddress inet = incomingConnexion.getInetAddress();
			    	String ip = inet.toString().split("/")[1];
					
					peer.player.ip = ip;
					peer.player.port = incomingConnexion.getPort();
					connectionEstablished = true;
					peer.setConnection( new Connection(incomingConnexion));
					} catch (IOException ioe) {
					}
			    }
			} catch (IOException ioe) {	
				System.err.println("[O NOES ! I DOESNT CAN WAIT FOR CONNEKSHION ON STARBOARD. I CANNOT HAZ SERVRSOCKET =(]\n" + ioe);
			}
		}

		System.out.println("!!! I CAN HAZ CONNEKSHIUN AT " + role + " WIF " + peer.player.name + " ON IP " + peer.player.ip + " WIF PORT " + peer.player.port + "!!!");
	}

	/*
	 * Main process, initializing asynchronous establishment of peer to peer connection and handling the reception of peer messages 
	 */
	public void run() {
		try {
			initializesConnexion();
			introductions();
		} catch (Exception e) {
			return;
		}
		
		Runnable doWorkRunnable = new Runnable() {
		    public void run() { peer.game.tryStart();}
		};
		SwingUtilities.invokeLater(doWorkRunnable);
		
		String line;
	    while (peer.active) {
    		line = getIn();
	    	// Beginning of the game
    		if (line == null) {}
    		else if (line.equals("MAH BORD LOOKZ LIEK DIS")) {
    			receiveBoard();
    		}
			else if (line.length() > 55 && line.substring(0,43).equals("YO DAWG, I HEARD U LIEK PENALTIEZ! HERE IZ ")) {
				String sline = line.substring(43);
				int space = sline.indexOf(' ');
				
				if (sline.substring(space).equals(" ONES 4 U!1!")){
					int pNumber = Integer.parseInt(sline.substring(0,space));
					peer.game.takePenalties(pNumber);
				}
    		}
			else if (line.length() > 45 && line.substring(0,25).equals("I LOST TEH GAME, ALL YUR ")) {
				line = line.substring(25);
				int space = line.indexOf(' ');

				if (! line.substring(0,space).equals(String.valueOf(role))){ 
					continue;
				}
				line = line.substring(space);
				
				if (! (line.length() > 15 && line.substring(0,15).equals(" ARE BELONG TO "))) {
					continue;
				}
				line = line.substring(15);
				String[] split = line.split(":");
				String ip = split[0];
				
				line = split[1];
				
				space = line.indexOf(' ');
				int port = Integer.parseInt(line.substring(0,space));
				line = line.substring(space);

				if (! line.equals(" NAO")) {
					continue;
				}
				
				peer.player.ip = ip;
				peer.player.port = port;

				try {
					if (role == PlayerInteraction.Role.LARBOARD) {
						peer.serverSocket.close();
					}
					initializesConnexion();
					introductions();
				} catch (Exception e) {
					return;
				}
    		}
    		
	    
	    }
		
	}

	/*
	 * Handles the introductions and exchange of names 
	 */
	private void introductions() throws Exception {
		 // we initialize : we introduce ourselves
		if (role == PlayerInteraction.Role.STARBOARD) { 
			peer.introduction();

			String line = null;
			while (! (line != null && line.length() >= 47 && line.substring(0,47).equals("GRT! ME WUZ WAITIN 4 U TO PLAY WIF!! MAH NAYMZ "))) {
				if (! peer.active){
					throw new Exception("Connekshiun shut down");
				}
				line = peer.getIn();
			}

			peer.player.name = line.substring(47);
		}
		
		 // we receive the connection
		else {
			String line = null;
			while (!(line != null && line.length() >= 14 && line.substring(0,14).equals("HOHAI!!! I IZ "))) {
				if (! peer.active){
					throw new Exception("Connekshiun shut down");
				}
				line = peer.getIn();
			}
			peer.player.name = line.substring(14);
			
			peer.answerIntroduction();
		}

		System.out.println("I IZ INTRODUCED WIF " + peer.player.name + " ON " + role);
		peer.isReady = true;
		
	}

	/*
	 * Reads the content of the buffer (waiting for it to be non-null)
	 */
	private void receiveBoard() {
		String line = getIn();
		final Piece.Type[][] board = new Piece.Type[Board.height][Board.width];
		int y=0;
		while (! line.equals("") && line != null) {
			
			for (int i=0;i<line.length();i++){
				Piece.Type p = Piece.Type.empty;

	            switch (line.charAt(i)) {
		            case 'i' :  p = Piece.Type.I;  break;
		            case 'j' :  p = Piece.Type.J;  break;
		            case 'l' :  p = Piece.Type.L;  break;
		            case 'o' :  p = Piece.Type.O;  break;
		            case 's' :  p = Piece.Type.S;  break;
		            case 't' :  p = Piece.Type.T;  break;
		            case 'z' :  p = Piece.Type.Z;  break;
		            case 'p' :  p = Piece.Type.P;  break;
		            default: break;
	            }
	            
	            board[y][i] = p;
			}
			y++;
			line = getIn();
		}		
		
		peer.kthxbye();
		
		Runnable doWorkRunnable = new Runnable() {
		    public void run() {  peer.game.updateNeighbourBoard(peer.getRole(), board); }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
	}
	
	/*
	 * Reads the content of the buffer (waiting for it to be non-null)
	 */
	private String getIn() {
		return peer.getIn();
	}

}
