import java.awt.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.text.*;

/*
 * Contains the list of the players in the current game
 */
public class PlayersList extends JPanel {
	private static final long serialVersionUID = 5947600302620053751L;
	
	// Class parameters
	private Vector<Player> players = new Vector<Player>();
	private JTextPane display = new JTextPane();

	/*
	 * Initializes the module and graphical component
	 */
	public PlayersList () {
		this.add(display);
	    this.setOpaque(false);
	    display.setEditable(false);	
	    display.setPreferredSize(new Dimension(150,300));
	}
	
	/*
	 * Find a given player based on its ip:port
	 * If none is found, returns an unnamed player with ip:port
	 */
	public Player findPlayer(String ip, int port) {
		int n = players.size();

        for(int i=0;i<n;i++)
        {
        	Player p = players.get(i);
        	
        	if (p.ip.equals(ip) && p.port == port){
    			return p;
        	}
        }
        
        return new Player ("", ip, port);			
	}
	
	/*
	 * Returns the leader of the game, the first person in the list
	 */
	public Player leader() {
		return players.firstElement();
	}
	
	/*
	 * Returns whether or not the client is the leader 
	 */
	public boolean isLeader() {
		return players.firstElement().name.equals(Client.name);
	}

	/*
	 * Updates the display with a different list
	 */
	private void displayList() {
		// Resets the current display
		display.setText("");
		
		Document doc = display.getDocument();
		int n = players.size();

        for(int i=0;i<n;i++)
        {
        	Player p = players.get(i);
        	
        	String text = p.name + "\n";
        	SimpleAttributeSet style = new SimpleAttributeSet();

        	// Me is in bold blue
        	if (p.name.equals(Client.name)){
        	    StyleConstants.setBold(style, true);
        	    StyleConstants.setForeground(style, Color.BLUE);
        	}
        	// the leader is in pink with a kirby
        	if (i == 0){
        		text = "(>*-*)> " + text;
        	    StyleConstants.setForeground(style, Color.MAGENTA);
        	}
        	// actually insert the new line
    		try {
    			doc.insertString(doc.getLength(), text, style);
    		} catch (BadLocationException e) {
    			System.err.println("[NOES CAN PRINT IN TEXTBLOK =(]\n" + e);
    		}
        }
	}
	
	/*
	 * Updates the list of players thanks to a vector
	 */
	public void updateList(Vector<Player> list) {
		players = list;
		displayList();
	}
}
