import java.io.IOException;
import javax.naming.NameAlreadyBoundException;

/**
 * 
 * Manages the initial interaction with the server
 *
 */
public class ServerInteraction {
	// Server connection parameters
	public static String host = "localhost";
	public static int serverPort = 4242;
	
	// Connected elements
	private Connection connection;
	public PlayersList players;
	public Game game;
	public Screen screen;
	
	// Logical elements
	 private ServerMonitor monitor;

	/*
	 * Creates a ServerInteraction with the different components it interacts with
	 */
	public ServerInteraction(Screen _screen, Game _game, PlayersList _playersList){
		screen = _screen;
		game = _game;
		players = _playersList;
	}
	
	/**
	 * Creates the socket used by the server using object variables settings 
	 * May return either an NameAlreadyBoundException or an IOException if the connection to the server is impossible
	 */
	public synchronized int initialize() throws Exception {
		// Connection creation
		if (connection == null || ! connection.isInitialized()){
			connection = new Connection(host, serverPort);
		}
			
		// Introduction and name handling
        connection.send("HOHAI!! I IZ IN UR GAME, LISTENIN ON " + Client.port + ", MAH NAYMZ " + Client.name);
        String line = connection.getIn();

        if (line.equals("ALL UR NAMZ R BELONG TO SUMONE ELSE!!!")) {
           	throw new NameAlreadyBoundException();
        }
        else if (! line.equals("WELCUM " + Client.name + ", PREPARE YRSELF TO ENTER TEH TETRIS!!")) {
           	throw new IOException("Invalid answer from the server");
        }

        System.out.println("!!!! I CAN HAZ CONNEKSHIUN TO SERVR !!!!\n");

        // Creation of an event monitor    		
        monitor = new ServerMonitor(this);
        Thread monitorThread = new Thread(monitor);
        monitorThread.setName("MemeTris - Server monitor");
        monitorThread.setPriority(Thread.MIN_PRIORITY);
    	monitorThread.start();
        return 1;	
	}
		
	/**
	 * Sends a chat message
	 */
	public void chat(String msg) {
		connection.send("I SAYZ " + msg);	
	}
	
	/**
	 * Pings the server
	 */
	public void ping() {
		connection.send("I CAN HAZ A PONG?");
	}
	
	/**
	 * Attempts to start the game
	 */
	public void tryStart() {
		connection.send("I CAN HAZ A TETRIS!!!");		
	}

	/**
	 * Notifies the server that a message has been received
	 */
	public void ok() {
		connection.send("KTHXBYE");	
	}
	
	/*
	 * Closes the socket connection
	 */
	public void close() {
		System.err.println("I IZ LEAVIN SERVR ALONE ! MEEBEE I CAN HAZ TRANSMISHUN ERROR BUT I EATED TEHM !!!");
		monitor.stop();
		//screen.displayAuth();
		connection.close();
	}

	/*
	 * Gets an incoming message or a null message if the buffer is empty
	 */
	public String getIn() {
		return connection.getIn();
	}
}
