import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;

/*
 * Thread handling reception of server messages and timeout timers.
 * Uses a ServerInteraction
 */
public class ServerMonitor implements Runnable, ActionListener {
	// Timer settings
	private int ServerTimeOut = 5000;
	private int PingTimeOut = 1000;
	
	// Monitoring parameters
	private ServerInteraction server;
	private Timer pingTimer;
	private Timer serverTimer;
	private boolean activated = true;
	
	/**
	 * Creates a server monitor based on a serverInteraction
	 */
	public ServerMonitor(ServerInteraction serverConnection) {
		server = serverConnection;
		
		pingTimer = new Timer(PingTimeOut,this);
		pingTimer.start();
		serverTimer = new Timer(ServerTimeOut,this);
		serverTimer.start();
	}

	/*
	 * Main process, handling the reception of any message from the server
	 */
	public void run() {
		String line = "";
    	
	    while (activated) {
    		line = getIn();
		    	
	    	// Beginning of the game
    		if (line == null) {}
    		else if (line.equals("TEH TETRIS WILL START SOON")) {
				System.out.println("OMG ! ITS A DOUBLE SERVER SIGNAL ! TEH TETRIS IS ABOUT TO START !!!\n");
					
	    		line = getIn();
			    	
				if (line.length() > 7 && line.substring(0,7).equals("SEED R ")) {
					server.game.seed = Integer.parseInt(line.substring(7));    	    			
				}
				else {
					System.err.println("[BATMAN RIDES SUPERMAN, TEH RANDOM SEED IS INVALID]");
					System.exit(1);
				}

					line = getIn();
	
				try {
					String[] split = line.split(" ");
					if (split.length == 10&& split[0].equals("U")  && split[1].equals("PLAYS") && split[2].equals("WIF") && split[4].equals("AT") && split[5].equals("STARBOARD") && split[6].equals("N") && split[8].equals("AT") && split[9].equals("LARBOARD")) {
						String[] sb = split[3].split(":");
						String[] lb = split[7].split(":");
						
						if (sb.length == 2 && lb.length == 2) {
			    			server.game.p_starboard = new PlayerInteraction(server.players.findPlayer(sb[0],Integer.parseInt(sb[1])), PlayerInteraction.Role.STARBOARD, server.game);				    			
						}
						else {
			    			throw new Exception();
						}
					}
					else {
						throw new Exception();
					}
				}
				catch(Exception e) {
					System.err.println("[NO CAN READZ INFORMASHUN ABOUT UR NEIGHBOURS, U CANT PLAY NAOW]");
					System.exit(1);
				}
				activated = false;
	
				Runnable doWorkRunnable = new Runnable() {
				    public void run() { server.screen.displayGame(); }
				};
				SwingUtilities.invokeLater(doWorkRunnable);
				
				server.ok();
			}
	
	    	// Players list reception
			else if (line.equals("PLAYERZ LIST R:")) {
				receivePlayersList();
			}
				
			// Server ping reception
			else if (line.equals("LOLYES")) {
				serverTimer.restart();
			}
				
			// Chat reception
			else if (line.length() > 13 && line.substring(0,13).equals("WUT PPL SAYZ ")) {
				String sline = line.substring(13);
				final String[] split = sline.split("\t");
	
					Runnable doWorkRunnable = new Runnable() {
				    public void run() { server.screen.addChatLine(split[0], split[1]); }
				};
				SwingUtilities.invokeLater(doWorkRunnable);
			}
	    }
	}
	
	/*
	 * Handles the reception of the players list
	 * /!\ manipulates graphical elements , must be InvokedLater
	 */
	private void receivePlayersList() {
		try {
			final Vector<Player> list = new Vector<Player>();	    	
	    	
			System.out.println("! RCVIN PLAYERZ LISTS");
			String line = server.getIn();
	    	
			while (line == null || ! line.equals("")){
				if (line != null) {
					String[] split = line.split("\t|:");
					list.add(new Player (split[0], split[1], Integer.parseInt(split[2])));	
				}
				
	    		line = server.getIn();
			}
			

			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	server.players.updateList(list);
			    	server.screen.refreshLeadership();
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);
			
		}
		catch (Exception e){
			System.err.println("[I DOESNT CAN RECV MSG OF PLAYERZ LIST =(]\n" + e);
			e.printStackTrace();
			System.exit(1);
		}
	}

	/*
	 * Stops all timer so that the thread can be garbage-collected
	 */
	public void stop() {
		pingTimer.stop();
		serverTimer.stop();
		activated = false;
	}

	/*
	 * Action listener dealing with the timers
	 */
	public void actionPerformed(ActionEvent e) {
		// Ping timer
		if (e.getSource() == pingTimer) {
			server.ping();
	    }
		
		// Server timeout timer
		if (e.getSource() == serverTimer) {
			server.close();
	    }
	 }
	
	/*
	 * Reads the content of the buffer (waiting for it to be non-null)
	 */
	private String getIn() {
		return server.getIn();
	}
}
