import java.net.*;

/**
 * Represents the interaction between the server and a client
 */
public class ClientInteraction {
	// Parameters accessed by ClientsList. We could do getter and setter so that they're readonly
	public String name = "unknown";
	public String ip;
	public int port;
	
	// Logical components
	private Connection connection;
	private ClientsList clientsList;
	
	/**
	 * Constructs the interaction by creating the connection and the monitor
	 */
	public ClientInteraction(Socket socket, ClientsList _clientsList) {
		connection = new Connection(socket);
		clientsList = _clientsList;
		
		InetAddress inet = socket.getInetAddress();
		ip = inet.toString().split("/")[1];

		System.out.println("** i haz a new client request : ");

        // Creation of an event monitor   
   		ClientMonitor monitor = new ClientMonitor(this, clientsList);
   	    Thread monitorThread = new Thread(monitor);
   	    monitorThread.setName("MemeTris Server - Client monitor for ip " + ip);
        monitorThread.setPriority(Thread.MIN_PRIORITY);
    	monitorThread.start();            
	}
	
	/** 
	 * Ends the connexion with the client
	 */
	public void die() {
        System.out.println("** " + name + " AZ LEFT TEH TETRIS =(\n");	
        clientsList.removeClient(this);		
		connection.close();	
	}
	
	/**
	 * Sends a text line to the client
	 */
	synchronized public void send(String s){
    	connection.send(s);
	}

	/** 
	 * Sends a message notifying a wrong command
	 */
	public void notCompute(){
		send("DOES NOT COMPUTE. WUT DYA MEEN?");
	}
	
	/** 
	 * Sends a message notifying a wrong authentification
	 */
	public void unAuthentified(){
		send("DUNNO WHO JOO IZ, WHO R U?");
	}

	/*
	 * Gets an incoming message or a null message if the buffer is empty
	 */
	public String getIn() {
		return connection.getIn();
	}
}
