import java.awt.event.*;
import javax.swing.Timer;

public class ClientMonitor implements Runnable, ActionListener {
	// Timeout timer
	private Timer timer = new Timer(Server.serverTimeOut,this);
	
	// Logical components
	private ClientInteraction client;
	private ClientsList clientsList;
	private boolean activated = true;

	/**
	 * Creates a client monitor attached to a given client (connection)
	 */
	public ClientMonitor(ClientInteraction _client, ClientsList _clientsList) {
		client = _client;      
		clientsList = _clientsList;
        System.out.println("monitoring inishializd on " + client.ip + "\n");
	}
	
	/*
	 * Deals with introduction (authentification) of the client and then answers its messages
	 */
	public void run() {
		String line = "";

        // Introduction and name handling
	    client.name = "";
	    while (client.name.equals("")) {
            line = getIn();
	        String[] split = line.split(",");
	        	
	        // Verifies that we receive an introduction message
	        if (! (split.length == 3 && split[0].equals("HOHAI!! I IZ IN UR GAME") &&
	        		split[1].length() > 13 && split[1].substring(0,13).equals(" LISTENIN ON ") &&
	    			split[2].length() > 11 && split[2].substring(0,11).equals(" MAH NAYMZ "))) 
	        {
	        	client.unAuthentified(); // If not, send back a message to notify
	        }
	        else
	        {
	        	// Verifies that the name is available
		        try {
		        	client.port = Integer.parseInt(split[1].substring(13));
		        	client.name = split[2].substring(11);
			        if (! clientsList.isNameAvailable(client.name)) {
			           	client.send("ALL UR NAMZ R BELONG TO SUMONE ELSE!!!");
			           	client.name = "";
			        }
		        }
		        	
		        // Verifies the integrity of the port field (int) which throw raise an exception
		    	catch (NumberFormatException nfe){
		    		System.err.println("[SILLY STRING, U IZ NOT A INT, PORTS R FOR INT !!]\n" + nfe);
		    		client.unAuthentified();
		    		client.name="";
		    	}
	    	}
	    }      
	        	
	    // Authentification complete : we can add it to the clientslist and start watching for the timeout 
	    client.send("WELCUM " + client.name + ", PREPARE YRSELF TO ENTER TEH TETRIS!!");
	    System.out.println("new client ip r '" + client.ip + "' : " + client.port + " and name r '" + client.name + "'\n");
	    clientsList.newClient(client);
	    timer.start();
	        	
	    // Answers incoming messages
	    while (activated) {
	      	line = getIn();
	      	
	      	// Pings
	       	if (line.equals("I CAN HAZ A PONG?")) {
	       		client.send("LOLYES");
	            timer.restart();
	       	}
	    	   	
	       	// Chat
	       	else if (line.length() > 7 && line.substring(0,7).equals("I SAYZ ")) {
	       		clientsList.broadcast("WUT PPL SAYZ " + client.name + "\t" + line.substring(7));
	       	}
	       	
	       	// Quit
	       	else if(line.equals("ME CHOOSE TEH RED PILL, KTHXBYE")) {
	       		die();
	       	}
	       	
	       	// Try to start
	       	else if(line.equals("I CAN HAZ A TETRIS!!!")) {
	       		if (! clientsList.tryStart(client)) {
	       			// If the client doesn't have the right to start, it is considered as an unknown command
	       			client.notCompute();
	       		}
	       	}
	       	else if(! line.equals("")){
	       		client.notCompute();
	       	}
	    }
	}
	
	/*
	 * Kills everything about this client so it can be garbage collected
	 */
	private void die() {
		client.die();
		timer.stop();
		activated = false;
	}

	/*
	 * Reads the content of the buffer (waiting for it to be non-null)
	 */
	private String getIn() {
		return client.getIn();
	}

	/*
	 * Timeout handler : kicks the client if it doesnt ping regularly
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == timer) {
	        System.out.println("-> i iz going to kik " + client.name + " koz i cannot haz a ping \n");
	        die();
	    }
	 }
}
