import java.util.Random;
import java.util.Vector;

/*
 * Manages the list of clients and general interactions with all of them
 */
public class ClientsList {
	private Vector<ClientInteraction> clients = new Vector<ClientInteraction>();

	/** 
	 * Verifies that a player name is available
	 */
	public boolean isNameAvailable(String name){
		boolean free = true;

        for(int i=0;i<clients.size();i++)
        {
        	if (name.equals(clients.get(i).name)) {
        	free = false;
        	}
        }		
		return free;
	}
	
	/**
	 * Handles the connection of a client, adding it to the overwatched clients
	 */
	public void newClient(ClientInteraction client){
		clients.add(client);
		broadcastPlayersList();
	}

	/**
	 * Handles the disconnction of a client, removing it from the overwatched clients
	 */
	public void removeClient(ClientInteraction client){
		clients.remove(client);
		broadcastPlayersList();
	}
	
	/**
	 * Sends a mesage to all clients
	 */
	public void broadcast(String message){
        for(int i=0;i<clients.size();i++)
        {
        	clients.get(i).send(message);
        }		
	}
	
	/**
	 * Broadcasts client list to all clients currently connected
	 */
	public void broadcastPlayersList(){
		String list = "PLAYERZ LIST R:\r\n";
        for(int i=0;i<clients.size();i++)
        {
        	list += clients.get(i).name + "\t" + clients.get(i).ip + ":" + clients.get(i).port + "\r\n";
        }
        broadcast(list);
    }

	/**
	 * Verifies the authorization of client to start the game
	 */
	public boolean tryStart(ClientInteraction client) {
		if (client.equals(clients.firstElement())){
			startGame();
			return true;
		}		
		else {
			return false;
		}
	}

	/**
	 * Actually starts the game
	 */
	private void startGame() {
		Random rand = new Random(); 
		
		String message = "TEH TETRIS WILL START SOON\n";
		message += "SEED R " + Math.abs(rand.nextInt() % 32003) + "\r\n"; //  % 6587493

		int n = clients.size();
		
		// Tells every client about their neighbours
        for(int i=0;i<n;i++)
        {
        	ClientInteraction starboard = clients.get((i + n - 1) % n);
        	ClientInteraction larboard = clients.get((i + 1) % n);
        	String clientMessage = message;
        	clientMessage += "U PLAYS WIF " + starboard.ip + ":" + starboard.port + " AT STARBOARD N ";
        	clientMessage += larboard.ip + ":" + larboard.port + " AT LARBOARD";
        	clients.get(i).send(clientMessage);
        }
	}
}
