import java.io.*;
import java.net.*;

/*
 * Manages any network socket at a slightly higher level
 */
public class Connection {
	
	// Connection settings
	private static int connectionTimeOut = 1000;
	
	// Connection parameters
	private String ip;
	private int port;
	private Socket socket;

	// I/O buffers
	private BufferedReader in;
	private OutputStreamWriter out;
	
	/*
	 * Creates a connection from an existing socket (for instance for a server)
	 */
	public Connection (Socket _socket) {
		socket = _socket;

		InetAddress inet = socket.getInetAddress();
		ip = inet.toString().split("/")[1];
		port = socket.getPort();
		initializeBuffers();
	}

	/*
	 * Creates a connection to the given ip (creates the socket)
	 */
	public Connection (String _ip, int _port) {
		ip = _ip;	
		port = _port;	
		
	    try {
			socket = new Socket(ip, port);
			socket.setSoTimeout(connectionTimeOut);
		} catch (Exception e) {
			System.err.println("[I DOESNT CREATE RITE SOCKET =(]\n" + e);
			e.printStackTrace();
		}	
		initializeBuffers();
	}

	/* 
	 * Creates I/O buffers from the socket to ease the communication
	 */
	synchronized private void initializeBuffers() {
	    try {
			InputStream inBuffer = socket.getInputStream();
	        in = new BufferedReader(new InputStreamReader(inBuffer));
			out = new OutputStreamWriter(socket.getOutputStream());	
		} catch (Exception e) {
			System.err.println("[I DOESNT CREATE RITE SOCKET I/O BUFFER =(]\n" + e);
			e.printStackTrace();
		}	
	}

	/*
	 * Indicates whether the socket is successfully open or not.
	 */
	public boolean isInitialized() {
		return (socket != null); // || ! socket.isConnected()
	}
	
	/*
	 * Sends a one-line message through the buffer
	 */
	synchronized public void send(String s) {
	    try {
			out.write(s + "\n");
			out.flush(); 
		} catch (IOException e) {
			System.err.println("[I DOESNT CAN TRANSMIT MSG TO " + ip + " =(]\n" + e);
		}
	}

	/*
	 * Gets the incoming buffer content (null or message)
	 */
	public String getIn() {
		String s = null;
		try {
			s = in.readLine();
		} 
		catch (SocketTimeoutException e) {
		}
		catch (IOException e) {
			System.err.println("[I DOESNT CAN RECV MSG FROM " + ip + " =(]\n" + e);	
		}
		return s;
	}
	
	/*
	 * Closes the connection
	 */
	synchronized public void close() {
		try {
			socket.close();
			socket = null;
		} catch (Exception e) {
			System.err.println("[I DOESNT CAN CLOSE TEH SOCKT]\n" + e);
		}	
	} 
	
	/*
	 * Allows other connections to use the same timeout : only one parameter to change
	 */
	public static int getTimeout (){
		return connectionTimeOut;
	}
}
